@@ -84,11 +84,12 @@ sub new {
 	return $self
 }
 
-sub DESTROY {
-	# wrap around eval. 
-	# destroying of fields in perl5.8 cleanup can cause strange errors, where
-	# it complains, that it cannot coerce array into hash
-	eval { DEBUG( 100,"DESTROY context $_[0] callid=$_[0]->{callid}" ) };
+# destroying of fields in perl5.8 cleanup can cause strange errors, where
+# it complains, that it cannot coerce array into hash. So use this function
+# on your own risks and rename it to DETSTROY if you want to have debugging
+# info
+sub _DESTROY {
+	DEBUG( 100,"DESTROY context $_[0] callid=$_[0]->{callid}" );
 }
 
 ############################################################################
@@ -269,10 +270,11 @@ sub handle_response {
 	# if response does not terminates transaction one need to add
 	# it again
 	my $tid = $response->tid;
+	my $method = $response->method;
 	my $trans = $self->{_transactions};
 	my (@ntrans,$tr);
 	foreach my $t (@$trans) {
-		if ( !$tr and $t->{tid} eq $tid ) {
+		if ( !$tr and $t->{tid} eq $tid and $method eq $t->{request}->method) {
 			$tr = $t;
 		} else {
 			push @ntrans,$t
@@ -295,18 +297,6 @@ sub handle_response {
 		return;
 	};
 
-
-	# check if it's a response to the current method in the
-	# transaction (ACK,CANCEL,INVITE share the same transaction,
-	# but are different methods)
-
-	my $method = $tr->{request}->method;
-	$response->cseq =~m{^\d+\s+(\w+)};
-	if ( $method ne $1 ) {
-		DEBUG( 10,"got response to method $1 but current method is $method. DROP" );
-		return;
-	}
-
 	my $cb = $tr->{callback};
 	my @arg = ($endpoint,$self);
 	my $code = $response->code;
@@ -360,16 +350,20 @@ sub handle_response {
 			# is response to INVITE, create ACK
 			# and propagate to upper layer
 			my $req = $tr->{request};
-			if ( my $contact = $response->get_header( 'contact' )) {
-				# 12.1.2 - set URI for dialog to contact given in response which
-				# establishes the dialog
-				$contact = $1 if $contact =~m{<(\w+:[^>\s]+)>};
-				$req->set_uri( $contact );
-			}
+
+			# create ACK, the ACK needs to be send with the original URI,
+			# even if the contact header (see below) changes the dialogs URI
 			my $ack = $req->create_ack( $response );
 			invoke_callback($cb,@arg,0,$code,$response,$leg,$from,$ack);
 			$endpoint->new_request( $ack,$self,undef,undef,leg => $leg, dst_addr => $from );
 
+			# 12.1.2 - set URI for dialog to contact given in response which
+			# establishes the dialog
+			if ( my $contact = $response->get_header( 'contact' )) {
+				$contact = $1 if $contact =~m{<(\w+:[^>\s]+)>};
+				$req->set_uri( $contact );
+			}
+
 			# use to-tag from this request to update 'to'
 			# FIXME: this should probably be better done by the upper layer
 			# which decides, which call to accept (in case of call-forking with
